@@ -17,7 +17,7 @@
 %                                 May 2002                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -122,23 +122,11 @@ static Image *ReadCLIPBOARDImage(const ImageInfo *image_info,
   LPVOID
     clip_mem;
 
-  MagickBooleanType
-    status;
-
-  register ssize_t
-    x;
-
-  register Quantum
-    *q;
-
   size_t
     clip_size,
     total_size;
 
-  ssize_t
-    y;
-
-  unsigned int
+  unsigned char
     offset;
 
   void
@@ -152,8 +140,7 @@ static Image *ReadCLIPBOARDImage(const ImageInfo *image_info,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
   image=AcquireImage(image_info,exception);
-  if (!IsClipboardFormatAvailable(CF_BITMAP) &&
-      !IsClipboardFormatAvailable(CF_DIB) &&
+  if (!IsClipboardFormatAvailable(CF_DIB) &&
       !IsClipboardFormatAvailable(CF_DIBV5))
     ThrowReaderException(CoderError,"NoBitmapOnClipboard");
   if (!OpenClipboard(NULL))
@@ -178,6 +165,7 @@ static Image *ReadCLIPBOARDImage(const ImageInfo *image_info,
   if (clip_mem == (LPVOID) NULL)
     {
       CloseClipboard();
+      clip_data=RelinquishMagickMemory(clip_data);
       ThrowReaderException(CoderError,"UnableToReadImageData");
     }
   p=(unsigned char *) clip_data;
@@ -186,7 +174,23 @@ static Image *ReadCLIPBOARDImage(const ImageInfo *image_info,
   (void) GlobalUnlock(clip_mem);
   (void) CloseClipboard();
   memset(clip_data,0,BMP_HEADER_SIZE);
-  offset=((unsigned int) p[0])+BMP_HEADER_SIZE;
+  offset=p[0];
+  if ((p[0] == 40) && (p[16] == BI_BITFIELDS))
+    offset+=12;
+  else
+    {
+      unsigned int
+        image_size;
+
+      image_size=(unsigned int) p[20];
+      image_size|=(unsigned int) p[21] << 8;
+      image_size|=(unsigned int) p[22] << 16;
+      image_size|=(unsigned int) p[23] << 24;
+      /* Hack for chrome where the offset seems to be incorrect */
+      if (clip_size - offset - image_size == 12)
+        offset+=12;
+    }
+  offset+=BMP_HEADER_SIZE;
   p-=BMP_HEADER_SIZE;
   p[0]='B';
   p[1]='M';
@@ -340,7 +344,7 @@ static MagickBooleanType WriteCLIPBOARDImage(const ImageInfo *image_info,
       ThrowWriterException(CoderError,"UnableToWriteImageData");
     }
   clip_mem=GlobalLock(clip_handle);
-  if (clip_handle == (LPVOID) NULL)
+  if (clip_mem == (LPVOID) NULL)
     {
       (void) GlobalFree((HGLOBAL) clip_handle);
       clip_data=RelinquishMagickMemory(clip_data);
